

set VERSION "F2VB"

loadsource  sys.common

set frametitle "MIDAS Data Acquisition System Control (Afrodite)"

set CAMACPresent 1
set CAMACEnable 1

set VMEPresent 0
set VMEEnable 1

set VXIPresent 0
set VXIEnable 1

set TSPresent 1
set TSEnable 1

set txEnable 1

set histEnable 1
set HistMode 0

set RDOUTEnable 1


set DAProgram FERAExecV15

lappend setuplist [list "Force Spectrum Map needed"          map_needed]

lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]
lappend setuplist [list "VME data source configuration"    "Configure_VME_Data_Sources"]

lappend actionlist [list "Raw VME Data Display"  "Dump_Raw_VME_Data"]
lappend actionlist [list "Raw VXI Data Display"  "Dump_Raw_VXI_Data"]

set TestBuffer "ffff800b  7fff0100  00000000  00001930  70007000  700072af  7fff7fff  7fff7000  7000aa55"
set TestBufferLen 36    ;#  for single event mode

 
proc Custom_Start {} {
    global Options
    global VXIPresent VMEPresent

   if {[info exists Options(VMEPresent)]} {set VMEPresent $Options(VMEPresent)}
   if {[info exists Options(VXIPresent)]} {set VXIPresent $Options(VXIPresent)}

    update_Options

   if {$VMEPresent == 0} {set VMEEnable 0}
   if {$VXIPresent == 0} {set VXIEnable 0}

}

proc Setup_Rdout {} {
   global EGreport Options
   global Supported_Modules

   set-footer "Defining hardware"
   set-busy

   set EGreport_SAVE $EGreport
   set EGreport -1

   ClearConfig

#    first we define FERA data modules

   set datalist [inform camaccontrol inquire datalist]
   foreach item $datalist {
       set moduletype [first $item]
       set module [second $item]
       set vsn [fifth $item]
       switch $moduletype {
           {LeCroy 4300}    {define_lecroy_module $module $vsn}
           {Clover}         {define_clover_module $module $vsn}
           {FR48REG}        {define_fr48reg_module $module $vsn}
           {NACCFC}         {define_NACCFC_module $module $vsn}
           default          {insert-log "No definition for $module"}
       }
   }

#    then we define VME data modules

   foreach module $Supported_Modules {
       if {$Options(Include.$module) == 1} then {define_[set module]_config}
   }

   set EGreport $EGreport_SAVE

   clear-busy
   set-footer "All hardware defined"
}

proc create_spectra {} {
   global SpectrumServer EGreport Options
   global Supported_Modules

   set-footer "Creating spectra"
   set-busy

   set EGreport_SAVE $EGreport
   set EGreport -1

   Make_Xlate $SpectrumServer

   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat 0 0"

   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

#    first we define FERA data modules and create spectra

   set datalist [inform camaccontrol inquire datalist]
   foreach item $datalist {
       set moduletype [first $item]
       set module [second $item]
       set vsn [fifth $item]
       switch $moduletype {
           {LeCroy 4300}    {create_lecroy_spectra $module $vsn}
           {Clover}         {create_clover_spectra $module $vsn}
           {FR48REG}        {create_fr48reg_spectra $module $vsn}
           {NACCFC}         {create_NACCFC_spectra $module $vsn}
           default          {insert-log "No spectra created for $module"}
       }
   }

#    then we add VME data modules

   foreach module $Supported_Modules {
       if {$Options(Include.$module) == 1 &&  $Options(Histogram.$module) == 1} then {
           create_[set module]_spectra
       }
   }

   set EGreport $EGreport_SAVE

   clear-busy
   set-footer "Spectra created and mapped"
}

proc define_clover_module {module vsn} {DefineConfig "$vsn 5 1 0 0 0"}

proc create_clover_spectra {module vsn} {
   global SpectrumServer HistMode

   set adcnames [list HitPatt GeA GeB GeC GeD TacA TacB TacC TacD "" "" "" BGOSum BGOTac]
   set adclengths [list 32768 16384 16384 16384 16384 4096 4096 4096 4096 0 0 0 4096 4096]

#     RIS Corp Clover module

    if {$HistMode == 0} {return}

     for {set j 0} {$j <= 14} {incr j} {
         set adc [lindex $adcnames $j]
         set len [lindex $adclengths $j]
         if {$adc != ""} {

            set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $len :name '$module.$adc' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
            if {$z!=0 && $m==0x30004} then {set z 0}
            if {$z!=0} then {
               display-eg-error-report self $m "Attempting to create spectrum $module.$adc"
               clear-footer
               error $m
            } else {
               map_spectrum "$module.$adc $vsn $j"
 #insert-log "created $module.$adc for vsn=$vsn and sub address=$j"
            }
         }
    }
}

proc define_NACCFC_module {module vsn} {DefineConfig "$vsn 6 1 0xffff 0 0"}

proc create_NACCFC_spectra {module vsn} {
   global SpectrumServer

#     NAC CAMAC/FeraBus Controller

    for {set j 0} {$j <= 15} {incr j} {

     if {$j == 0} then {set adc HitPatt} else {set adc Ch$j}

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name '$module.$adc' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $module.$adc"
          clear-footer
          error $m
      } else {
          map_spectrum "$module.$adc $vsn $j"
 insert-log "created $module.$adc for vsn=$vsn and data word=$j"
      }
    }
}

proc Go_Electronics {} {
    global VXIPresent VXIEnable

    if {$VXIEnable == 0} {return}
    if {[inform vxicontrol VXImodule2crate SlotOne0] == "VXI"} {return}

#    there is a SlotOne module in this system - enable the front panel TTLTRG lines

    set vl [EG "register :name '/VXI/SlotOne0.Mode' ?value"]
    set vl [expr ($vl & 0xdf) + 0x20]    ;#   Enable front panel TTLTRG lines 
    EG "register :name '/VXI/SlotOne0.Mode' :value '$vl'"
}


loadsource sys.fera


